/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it
 * is applied to this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * AgroSense. If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.field.type.wizard;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Date;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import nl.cloudfarming.client.field.model.Field;
import nl.cloudfarming.client.field.type.FieldDataPanel;
import org.openide.util.NbBundle;

/**
 * Controller for field properties. Controls the {@link FieldDataPanel}
 * @author Merijn Zengers <Merijn.Zengers@gmail.nl>
 */
@NbBundle.Messages({"FieldController empty name=Name is required",
    "FieldController validity error message=Start date must be before end date",
    "FieldController empty start date=Start date is required",
    "FieldController emty end date=End date is required"})
public abstract class FieldController {

    private FieldDataPanel visualPanel;
    private final Field field;

    public FieldController(Field field) {
        this.field = field;
    }

    protected abstract void panelUpdated();

    protected FieldDataPanel getPanel() {
        if (visualPanel == null) {
            visualPanel = new FieldDataPanel(field);
            VisualPanelListener panelListener = new VisualPanelListener();
            visualPanel.getNameField().getDocument().addDocumentListener(panelListener);
            visualPanel.getValidityStartField().addActionListener(panelListener);
            visualPanel.getValidityStartField().getEditor().getDocument().addDocumentListener(panelListener);
            visualPanel.getValidityEndField().addActionListener(panelListener);
            visualPanel.getValidityEndField().getEditor().getDocument().addDocumentListener(panelListener);
        }
        return visualPanel;
    }

    protected Field getField() {
        return field;
    }

    /**
     * Is the panel valid Checks if the name is not null and is not empty Checks
     * if the start date is not null Checks if the end date is not null Checks
     * if the star date is smaller than end date.
     *
     * @return
     */
    protected boolean isPanelValid() {
        return isNameValid() && isValidityValid();
    }

    protected void validatePanel() {
        visualPanel.getNameRow().setValid();
        visualPanel.getValidityRow().setValid();
        if (!isNameValid()) {
            visualPanel.getNameRow().setInvalid(Bundle.FieldController_empty_name());
        }
        if (!isStartDateFilled()) {
            visualPanel.getValidityRow().setInvalid(Bundle.FieldController_empty_start_date());
        } else if (!isValidityValid()) {
            visualPanel.getValidityRow().setInvalid(Bundle.FieldController_validity_error_message());
        }
    }

    /**
     * Validate the name
     *
     * @param userInput
     */
    public boolean isNameValid() {
        String userInput = visualPanel.getNameField().getText().trim();
        return userInput != null && !userInput.isEmpty();
    }

    public boolean isStartDateFilled() {
        return visualPanel.getValidityStartField().getDate() != null;
    }

    /**
     * Validate the start and end date
     *
     * @param startDate
     * @param endDate
     * @return
     */
    public boolean isValidityValid() {
        Date startDate = visualPanel.getValidityStartField().getDate();
        Date endDate = visualPanel.getValidityEndField().getDate();
        return startDate != null && (endDate == null || startDate.before(endDate));
    }

    private class VisualPanelListener implements DocumentListener, ActionListener {

        @Override
        public void insertUpdate(DocumentEvent e) {
            panelUpdated();
        }

        @Override
        public void removeUpdate(DocumentEvent e) {
            panelUpdated();
        }

        @Override
        public void changedUpdate(DocumentEvent e) {
            panelUpdated();
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            panelUpdated();
        }
    }
}
