/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.field.type.wizard;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.AbstractAction;
import javax.swing.JComponent;
import nl.cloudfarming.client.field.model.Field;
import nl.cloudfarming.client.field.model.FieldManager;
import nl.cloudfarming.client.model.EnterpriseIdProvider;
import org.netbeans.api.project.Project;
import org.openide.DialogDisplayer;
import org.openide.WizardDescriptor;
import org.openide.util.Lookup;

/**
 *
 * @author Timon Veenstra <monezz@gmail.com>
 */
//@NbBundle.Messages(value = {"new treatmentzone group wizard title=New Treatmentzone Group"})
public class NewFieldAction extends AbstractAction implements ActionListener {

    private final Project project;
    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.field.type.wizard.NewFieldAction");

    public NewFieldAction(Project project) {
        this.project = project;
        putValue(NAME, Bundle.new_field_wizard_title());
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        List<WizardDescriptor.Panel<WizardDescriptor>> panels = new ArrayList<>();
        EnterpriseIdProvider eip = project.getLookup().lookup(EnterpriseIdProvider.class);
        assert eip != null : "Cannot initiate new field wizard without EnterpriseProvider in the lookup";
        Field newField = new Field(eip);
        panels.add(new NewFieldWizardPanel(newField));
        String[] steps = new String[panels.size()];
        for (int i = 0; i < panels.size(); i++) {
            Component c = panels.get(i).getComponent();
            // Default step name to component name of panel.
            steps[i] = c.getName();
            if (c instanceof JComponent) { // assume Swing components
                JComponent jc = (JComponent) c;
                jc.putClientProperty(WizardDescriptor.PROP_CONTENT_SELECTED_INDEX, i);
                jc.putClientProperty(WizardDescriptor.PROP_CONTENT_DATA, steps);
                jc.putClientProperty(WizardDescriptor.PROP_AUTO_WIZARD_STYLE, true);
                jc.putClientProperty(WizardDescriptor.PROP_CONTENT_DISPLAYED, true);
                jc.putClientProperty(WizardDescriptor.PROP_CONTENT_NUMBERED, true);
            }
        }
        WizardDescriptor wiz = new WizardDescriptor(new WizardDescriptor.ArrayIterator<>(panels));
        // {0} will be replaced by WizardDesriptor.Panel.getComponent().getName()
        wiz.setTitleFormat(new MessageFormat("{0}"));
        wiz.setTitle(Bundle.new_field_wizard_title());
        if (DialogDisplayer.getDefault().notify(wiz) == WizardDescriptor.FINISH_OPTION) {
            Field field = (Field) wiz.getProperty(NewFieldWizardPanel.PROP_FIELD);
            LOGGER.log(Level.FINEST, "retrieved newly created field from panel with id {0}", field.getItemIdNumber());
            FieldManager fm = Lookup.getDefault().lookup(FieldManager.class);
            fm.save(field);
        }
    }
}
